/* 
 * CVS identifier:
 * 
 * $Id: CodeStreamAnalyzer.java,v 1.3 2005-11-15 13:47:36 dengel Exp $
 * 
 * Class:                   CodeStreamAnalyzer
 * 
 * Description:             
 * 
 * Created on Mar 3, 2005
 * 
 * @author Dominik Engel <dengel@cosy.sbg.ac.at>
 * 
 * COPYRIGHT:
 * 
 * This software module was originally developed by Rapha�l Grosbois and
 * Diego Santa Cruz (Swiss Federal Institute of Technology-EPFL); Joel
 * Askel�f (Ericsson Radio Systems AB); and Bertrand Berthelot, David
 * Bouchard, F�lix Henry, Gerard Mozelle and Patrice Onno (Canon Research
 * Centre France S.A) in the course of development of the JPEG2000
 * standard as specified by ISO/IEC 15444 (JPEG 2000 Standard). This
 * software module is an implementation of a part of the JPEG 2000
 * Standard. Swiss Federal Institute of Technology-EPFL, Ericsson Radio
 * Systems AB and Canon Research Centre France S.A (collectively JJ2000
 * Partners) agree not to assert against ISO/IEC and users of the JPEG
 * 2000 Standard (Users) any of their rights under the copyright, not
 * including other intellectual property rights, for this software module
 * with respect to the usage by ISO/IEC and Users of this software module
 * or modifications thereof for use in hardware or software products
 * claiming conformance to the JPEG 2000 Standard. Those intending to use
 * this software module in hardware or software products are advised that
 * their use may infringe existing patents. The original developers of
 * this software module, JJ2000 Partners and ISO/IEC assume no liability
 * for use of this software module or modifications thereof. No license
 * or right to this software module is granted for non JPEG 2000 Standard
 * conforming products. JJ2000 Partners have full right to use this
 * software module for his/her own purpose, assign or donate this
 * software module to any third party and to inhibit third parties from
 * using this software module for non JPEG 2000 Standard conforming
 * products. This copyright notice must be included in all copies or
 * derivative works of this software module.
 * 
 * Copyright (c) 1999/2000 JJ2000 Partners.
 * 
 * 
 * 
 */

package jj2000.j2k.codestream.reader.analyze;

import java.io.EOFException;
import java.io.IOException;
import java.util.Vector;

import jj2000.j2k.codestream.Markers;
import jj2000.j2k.io.RandomAccessIO;
import jj2000.j2k.util.FacilityManager;
import jj2000.j2k.util.MsgLogger;

public class CodeStreamAnalyzer {
	
	 private RandomAccessIO in;
	 
	 private int savedPos;
	  
	/**
	 * 
	 */
	public CodeStreamAnalyzer(RandomAccessIO in) {
				this.in = in;
	}
	
	private void savePos()
	{
		try
		{
			savedPos = in.getPos();
		} catch (IOException e) {
			FacilityManager.getMsgLogger().printmsg(MsgLogger.WARNING,
					"An Exception has occured (savepos): " + e.toString());			
		}
	}
	
	private void restorePos()
	{
		try
		{
			in.seek(savedPos);
		
		} catch (IOException e) {
		FacilityManager.getMsgLogger().printmsg(MsgLogger.WARNING,
				"An Exception has occured (restorepos): " + e.toString());			
		}
	}
	
	
	public void analyze () throws IOException 
	{
		
		savePos();
		String res = "";
		Vector packets = new Vector(100);
		Packet packet;
		
		res +=  "Analyzing WP bitstream\n" +
				 "======================\n";
		

		try {
			while (true) {
					skipToBeforeSOP();		
					packet = readPacket();
					packets.add(packet);
			}
		} 
		catch (EOFException eof) {
			//end of file reached, that's ok
		}
		catch (IOException e) {
			throw e;
		}
		
		res += "\t parsed " + packets.size() + " packets\n";
		
		//TODO now do some statistics on these packets, based on which 
		//we might be able to infer something about the undder-
		//lying WP structure
		
		restorePos();
		FacilityManager.getMsgLogger().printmsg(MsgLogger.INFO,res);
		
	}
	
	/**
	 * 
	 * reads a packet from the underlying source, and saves its data to an object
	 * 
	 * @return filled packet object
	 * @throws IOException
	 */
	private Packet readPacket() throws IOException
	{
		Packet packet = new Packet();
		
		byte sopArray[] = new byte[Markers.SOP_LENGTH];
		
		int pos = in.getPos();		
		
		in.readFully(sopArray, 0, Markers.SOP_LENGTH);

		int val;
		val = sopArray[0];
		val <<= 8;
		val |= sopArray[1];
		if (val != Markers.SOP) {			
			throw new Error("Corrupted Bitstream: Could not parse SOP "
					+ "marker !");
		}

		
		// Check if length is correct
		val = (sopArray[2] & 0xff);
		val <<= 8;
		val |= (sopArray[3] & 0xff);
		if (val != 4) {
			throw new Error("Corrupted Bitstream: Corrupted SOP " + "marker !");
		}

		// Check if sequence number if ok
		val = (sopArray[4] & 0xff);
		val <<= 8;
		val |= (sopArray[5] & 0xff);			
		
		packet.pkIdx = val;					
		
		skipToEPH();
		packet.header = new byte[in.getPos() - pos];
		
		in.seek(pos);
		
		//save the start of this package in the object
		packet.offset = pos;
		
		in.readFully(packet.header, 0, packet.header.length);
		

		pos = in.getPos(); 		//save current position
		int plen = 0;		
		try {
			skipToBeforeSOP();
			plen = in.getPos() - pos;
		}
		catch (EOFException eof) // end of file reached, no more SOPs
		{
			plen = in.length() - pos -2 ; //-2 for EOD Marker
			in.seek(pos + plen);			
			byte check[] = new byte[2]; 
			in.readFully(check, 0, 2);
			int checkval = 0;
			checkval = check[0];
			checkval <<= 8;
			checkval |= check[1];
			if (checkval != Markers.EOC)
			{
				FacilityManager.getMsgLogger().printmsg(MsgLogger.WARNING, "Could not parse EOC marker");
			}
				
		}
		
		
		packet.body = new byte[plen];
		in.seek(pos);		//restore position
		
		in.readFully(packet.body, 0, packet.body.length);
		
		return packet;
		
	}
	

	
	/** 
	 * set position to next EPH
	 * 
	 * @return skipped bytes 
	 */
	public String skipToEPH()
	{
		byte b;
		int val;
		String hdr = "";
		boolean control = false;
		try {
			//Markers.EPH;
			b = 0;
			val = 0;
			while (val != Markers.EPH) {
				b = in.readByte();
				
				hdr += Integer.toHexString(0xff & b)+ " ";				
				if (b == -1)
				{
					val = 0xff &  b;
					val <<= 8;
					control = true;
				}
				else			
				{
					if (control)
						val |= b;
					else
						control = false;
				}
				
			}
			
		} catch (IOException e) {
			FacilityManager.getMsgLogger().printmsg(MsgLogger.WARNING,
					"An Exception has occured: " + e.toString());
		}
		return hdr;
	}
	
	/** 
	 * set position just before next SOP
	 * 
	 * @return skipped bytes 
	 */
	public String skipToBeforeSOP()  throws IOException {
		byte b;
		int val;
		String body = "";
		b = 0;
		val = 0;
		boolean control = false;
		while (val != Markers.SOP) {
			b = in.readByte();
			body += Integer.toHexString(0xff & b) + " ";
			if (b == -1) {
				val = 0xff & b;
				val <<= 8;
				control = true;
			} else {
				if (control)
					val |= b;
				else					
					control = false;
			}
		}
		in.seek(in.getPos() - 2);
		return body;
	}


}




/*


public String printPacketInfo()
{
	
	String info;
	savePos();	
	info = printPacketHeader();
	info += printPacketBody();
	restorePos();		
	return info;		
}

private String printPacketHeader() {
	String hdr = "HEADER:";
	byte b;
	int val;
	
	try {
		//Markers.EPH;
		b = 0;
		val = 0;
		while (val != Markers.EPH) {
			b = in.readByte();
			
			hdr += Integer.toHexString(0xff & b)+ " ";				
			if (b == -1)
			{
				val = 0xff &  b;
				val <<= 8;
			}
			else			
			{
				val |= b;					
			}
			
		}
		
	} catch (IOException e) {
		FacilityManager.getMsgLogger().printmsg(MsgLogger.WARNING,
				"An Exception has occured: " + e.toString());
	}
	
	return hdr;
			
}

private String printPacketBody() {
	String body = "\nBODY INFO: ";
	byte b;
	int val;
	boolean control = false;
	int pos = 0;
	
	try {	

		//Markers.EPH;
		pos = in.getPos();
		b = 0;
		val = 0;
		while (val != Markers.SOP) {
			b = in.readByte();
			
			//body += Integer.toHexString(0xff & b)+ " ";
			
			if (b == -1)
			{
				val = 0xff &  b;
				val <<= 8;					
			}
			else 			
			{
				val |= b;
			}				
		}
		body += " (payload: " + (in.getPos() - pos - 2) + " byte)";
	}
	catch (EOFException eof)
	{
		//that's ok

	}
	catch (IOException e) {
		FacilityManager.getMsgLogger().printmsg(MsgLogger.WARNING,
				"An Exception has occured: " + e.toString());	
	}
	
			
	return body;
			
}


*/