package jj2000.j2k.entropy.encoder;

/**
 * RDResult stores the result of a rate distortion optimization step.
 */
public class RDResult {
	/** The Lagrangian cost. */
	public double cost;
	/** The distortion. */
	public double dist;
	/** The rate in bytes. */
	public double rate;
	/** The Lagrangian lambda. */
	public float lambda;
	/** The wavelet packet basis (structure), should be coded as 1-0-String. */
	public String wpsDesc;

	public RDResult() {
		cost = 0;
		dist = 0;
		rate = 0;
		wpsDesc = "";
	}

	public RDResult(double cRate, double cDist, float rdThresh) {
		rate = cRate;
		dist = cDist;

		if (!(cDist == Double.MAX_VALUE)) {
			cost = cDist + rdThresh * cRate;
		} else {
			cost = Double.MAX_VALUE;
		}
		lambda = rdThresh;
		wpsDesc = "";
	}

	public RDResult(RDResult rdj, String r) {
		cost = rdj.cost;
		dist = rdj.dist;
		rate = rdj.rate;
		lambda = rdj.lambda;
		wpsDesc = r;
	}
	
	public RDResult(double childrenRate, double nDist, double nCostJ,
			float rdThresh, String r) {
		cost = nCostJ;
		dist = nDist;
		rate = childrenRate;
		lambda = rdThresh;
		wpsDesc = r;
	}
}
