/* 
 * CVS identifier:
 * 
 * $Id: RandomGenerator.java,v 1.6 2005-11-15 13:58:34 dengel Exp $
 * 
 * Class:                   RandomGenerator
 * 
 * Description:             
 * 
 * Created on Feb 17, 2005
 * 
 * @author Dominik Engel <dengel@cosy.sbg.ac.at>
 * 
 * COPYRIGHT:
 * 
 * This software module was originally developed by Raphal Grosbois and
 * Diego Santa Cruz (Swiss Federal Institute of Technology-EPFL); Joel
 * Askelf (Ericsson Radio Systems AB); and Bertrand Berthelot, David
 * Bouchard, Flix Henry, Gerard Mozelle and Patrice Onno (Canon Research
 * Centre France S.A) in the course of development of the JPEG2000
 * standard as specified by ISO/IEC 15444 (JPEG 2000 Standard). This
 * software module is an implementation of a part of the JPEG 2000
 * Standard. Swiss Federal Institute of Technology-EPFL, Ericsson Radio
 * Systems AB and Canon Research Centre France S.A (collectively JJ2000
 * Partners) agree not to assert against ISO/IEC and users of the JPEG
 * 2000 Standard (Users) any of their rights under the copyright, not
 * including other intellectual property rights, for this software module
 * with respect to the usage by ISO/IEC and Users of this software module
 * or modifications thereof for use in hardware or software products
 * claiming conformance to the JPEG 2000 Standard. Those intending to use
 * this software module in hardware or software products are advised that
 * their use may infringe existing patents. The original developers of
 * this software module, JJ2000 Partners and ISO/IEC assume no liability
 * for use of this software module or modifications thereof. No license
 * or right to this software module is granted for non JPEG 2000 Standard
 * conforming products. JJ2000 Partners have full right to use this
 * software module for his/her own purpose, assign or donate this
 * software module to any third party and to inhibit third parties from
 * using this software module for non JPEG 2000 Standard conforming
 * products. This copyright notice must be included in all copies or
 * derivative works of this software module.
 * 
 * Copyright (c) 1999/2000 JJ2000 Partners.
 * 
 * 
 * 
 */

package jj2000.j2k.util.param;

import java.util.Random;
import java.util.StringTokenizer;

import jj2000.j2k.util.FacilityManager;
import jj2000.j2k.util.MsgLogger;
import jj2000.j2k.wavelet.Subband;
import jj2000.j2k.wavelet.SubbandStructure;

/**
 * @author dengel
 *
 */
public  class RandomGenerator {

	private Random random;
 
	private SubbandStructure ss;
	
	public final static int DF_LINEAR = 0;
	public final static int DF_SQUARE = 1; // x^2 
	public final static int DF_EXP = 2; //e^x, not usable, growing too fast!
	
	public boolean use_positive = true;
	
	public int denormalize_function = 0;
	
	public long seed = 0;

	private int intervals = 255;	
	
	public RandomGenerator()
	{
		super();
	}
	
	
	/**
	 * A RandomGenerator for providing parameter values
	 * 
	 * @param key seed
	 * @param intervals number of partitions to be used for denormalizing
	 */
	public RandomGenerator(String options) {
		super();		
		parseSpec(options);
		if (seed == 0) FacilityManager.getMsgLogger().printmsg(MsgLogger.WARNING, "Random param generator: Seed is 0.");			
		random = new Random(seed);		
		FacilityManager.getMsgLogger().printmsg(MsgLogger.INFO, "Creating temporary full subband structure to hold parameters");
		ss = new SubbandStructure(0,0,512,512,5,null, null,Subband.WT_FULL);
		assignValuesRecursive(ss);
	}
	
	private void assignValuesRecursive(SubbandStructure ss)
	{
		ss.param = new double[2];
		ss.param[0]= nextParameter();
		ss.param[1]= nextParameter();
		
		
		if (! ss.isNode) return;		
		
		assignValuesRecursive((SubbandStructure) ss.getLL());
		assignValuesRecursive((SubbandStructure) ss.getHL());
		assignValuesRecursive((SubbandStructure) ss.getLH());
		assignValuesRecursive((SubbandStructure) ss.getHH());
	}
	
	
	private float nextParameter()
	{		
		return denormalize(random.nextInt(intervals), intervals); 
	}
	
	/**
	 *  get the randomized parameter for the horizontal decomposition for subband (rl, sbIdx)
	 * 
	 * @param rl
	 * @param sbIdx
	 * @return
	 */
	public float getHParameter (int rl, int sbIdx)
	{		
		return (float) ((SubbandStructure) ss.findNodeIdxInTree(rl, sbIdx)).param[0];
	}

	/**
	 * get the randomized parameter for the vertical decomposition for  subband (rl, sbIdx)
	 * 
	 * @param rl
	 * @param sbIdx
	 * @return
	 */
	public float getVParameter (int rl, int sbIdx)
	{		
		return (float) ((SubbandStructure) ss.findNodeIdxInTree(rl, sbIdx)).param[1];
	}

	protected float denormalize (int key_val, int radix)
	{		
		
	/* the following can be used to retain old denormalization
	if (true)
	{
		return ParamKeyUtil.denormalize(key_val, radix);
	}*/

	//   4.6 (-6   .. -1.4) 
	//   4,6 ( 1.4 ..  6  )
	//   ----
	//   9.2
	
	if (use_positive)
	{		
		//linear split first
		//float val = (float) key_val / (float) radix  * 9.2f; // but not by introducing assumption wrg to val
		 
		float split = random.nextFloat();
		if (split < 0.5f)  
		{				
			return ( -1.4f - (apply_df(key_val, radix)*4.6f) ); // --> -6 .. -1.4
		}
		else
		{
			return ( 1.4f + (apply_df(key_val, radix)*4.6f) ); // --> 1.4 .. 6
		}
	}
	else
	{
		float val = (apply_df (key_val,radix)  * 4.6f);
		return ( -1.4f - val); //not (val-6) to keep the dense part of the distribution near the smaller values!
		//return (val -6f); 
	}
	
	}

	private float apply_df (int key_val, int radix)
	{
		switch (denormalize_function) {
		case DF_LINEAR:
			return (float) key_val / radix;
			
		case DF_SQUARE:
			return (float) (Math.pow((float) key_val, 2) / (float) (Math.pow((float) radix, 2))) ;
			
		case DF_EXP:
			return (float) (Math.exp((float) key_val) / Math.exp((float) radix));
	
		default:
			throw new IllegalStateException("Bug");
		}
	}
	
	private void parseSpec(String spec)
	{
		if (spec == null) return;
		
		StringTokenizer t = new StringTokenizer(spec, "dsip", true);
		
		while (t.hasMoreTokens())
		{
			String token = t.nextToken();
			char var = token.charAt(0);
			switch (var) {
			case 's':
				this.seed = Long.parseLong(t.nextToken());
				break;
			case 'd':
				this.denormalize_function = Integer.parseInt(t.nextToken());
				break;
			case 'p':
				this.use_positive = Integer.parseInt(t.nextToken()) != 0;
				break;
			case 'i':
				this.intervals = Integer.parseInt(t.nextToken());
				break;
			default:
				throw new IllegalArgumentException("Malformed specification string for WwpsRandom: " + spec);
			}
		}
		FacilityManager.getMsgLogger().printmsg(MsgLogger.INFO, "Random Wavelet Structure Parameters: " + this.toString() );
	}
	
	public String toString() {
		return "[Random param generator; seed=" + seed + ", bins=" + intervals + ", df=" + denormalize_function + ", use_pos=" + use_positive + "]";
	}
	


}
