/* 
 * CVS identifier:
 * 
 * $Id: WPStructureSpec.java,v 1.9 2007-06-11 14:27:44 dengel Exp $
 * 
 * Class:                   WPStructureSpec
 * 
 * Description:             
 * 
 * Created on Dec 6, 2004
 * 
 * @author Dominik Engel <dengel@cosy.sbg.ac.at>
 * 
 * COPYRIGHT:
 * 
 * This software module was originally developed by Rapha�l Grosbois and
 * Diego Santa Cruz (Swiss Federal Institute of Technology-EPFL); Joel
 * Askel�f (Ericsson Radio Systems AB); and Bertrand Berthelot, David
 * Bouchard, F�lix Henry, Gerard Mozelle and Patrice Onno (Canon Research
 * Centre France S.A) in the course of development of the JPEG2000
 * standard as specified by ISO/IEC 15444 (JPEG 2000 Standard). This
 * software module is an implementation of a part of the JPEG 2000
 * Standard. Swiss Federal Institute of Technology-EPFL, Ericsson Radio
 * Systems AB and Canon Research Centre France S.A (collectively JJ2000
 * Partners) agree not to assert against ISO/IEC and users of the JPEG
 * 2000 Standard (Users) any of their rights under the copyright, not
 * including other intellectual property rights, for this software module
 * with respect to the usage by ISO/IEC and Users of this software module
 * or modifications thereof for use in hardware or software products
 * claiming conformance to the JPEG 2000 Standard. Those intending to use
 * this software module in hardware or software products are advised that
 * their use may infringe existing patents. The original developers of
 * this software module, JJ2000 Partners and ISO/IEC assume no liability
 * for use of this software module or modifications thereof. No license
 * or right to this software module is granted for non JPEG 2000 Standard
 * conforming products. JJ2000 Partners have full right to use this
 * software module for his/her own purpose, assign or donate this
 * software module to any third party and to inhibit third parties from
 * using this software module for non JPEG 2000 Standard conforming
 * products. This copyright notice must be included in all copies or
 * derivative works of this software module.
 * 
 * Copyright (c) 1999/2000 JJ2000 Partners.
 * 
 * 
 * 
 */

package jj2000.j2k.wavelet;

import java.io.DataOutputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import jj2000.j2k.ModuleSpec;
import jj2000.j2k.util.ArrayUtil;
import jj2000.j2k.util.FacilityManager;
import jj2000.j2k.util.MsgLogger;
import jj2000.j2k.util.ParameterList;

/**
 * @author dengel
 *
 */
public class WPStructureSpec extends ModuleSpec {

	
	private byte[] defaultDecomposition;
	
	/** used to convey a change in resolution level for a random decomposition
	 * structure
	 */
	public int newWlev = -1;
	
	/**
	 * Constructs WPStructureSpec for the specified number of components
	 * @param nt
	 * @param nc
	 * @param type
	 * @param pl
	 */
	public WPStructureSpec(int nt, int nc, byte type,
			ParameterList pl) {
		
		super(nt, nc, type);
		
		//TODO make this work, for many components, also for Falpha
		
		if (pl.containsKey("Wwps") && pl.containsKey("WwpsRandom"))
		{
			throw new IllegalArgumentException("Cannot use both Wwps and WwpsRandom.");
		}
		
		defaultDecomposition = null;
		
		//check is we are giving the structure or just
		//parameters to the PNRG
		if (pl.containsKey("Wwps"))
		{
			defaultDecomposition = ArrayUtil.createBitArrayFromString(pl.getParameter("Wwps"));
		} else if (pl.getParameter("Wreadwpsfile") != null) {
			try {
				FacilityManager.getMsgLogger().printmsg(MsgLogger.INFO, "Reading decomposition structure from " + pl.getParameter("Wreadwpsfile"));
				defaultDecomposition = ArrayUtil.readBitArrayFromHexFile(pl.getParameter("Wreadwpsfile"));
			} catch (IOException ioe) {
				FacilityManager.getMsgLogger().printmsg(MsgLogger.ERROR, "Could not read wps from \"" + pl.getParameter("Wreadwpsfile")+ "\"");
				System.exit(1);
			}
		}
		else if (pl.containsKey("WwpsRandom"))
		{
			WPRandomStructureGenerator wpRandom = new WPRandomStructureGenerator(pl.getParameter("WwpsRandom"));
			SubbandStructure sbbs = wpRandom.createRandomStructure();
			newWlev = sbbs.resLvl;
			defaultDecomposition = sbbs.toBitPattern();		
			FacilityManager.getMsgLogger().printmsg(MsgLogger.INFO, "New bit pattern " + ArrayUtil.printBitArrayAsHex(defaultDecomposition));
		}
				
		// if requested, write structure to file as a hex string
		// if bba is used, structure does not exist yet and will be written at a later point
		if ( pl.containsKey("Wwritewpsfile") && !pl.containsKey("Wbbacf"))
		{
			String fname =  pl.getParameter("Wwritewpsfile");
			try {
				ArrayUtil.writeBitArrayToHexFile(pl.getParameter("Wwritewpsfile"), defaultDecomposition); }
			catch (IOException ioe) {
				FacilityManager.getMsgLogger().printmsg(MsgLogger.ERROR, "Could not write WPS to " + fname);
			}
			catch (NullPointerException ne) {			
					FacilityManager.getMsgLogger().printmsg(MsgLogger.ERROR, "Null pointer exception when trying to write WPS Structure. Possibly you attempted to write it too soon.");			
			}
			finally	{
				FacilityManager.getMsgLogger().printmsg(MsgLogger.INFO, "WPS written to " + fname);				
			}				
		}	
		
		setDefault(defaultDecomposition);						
	}
	
	
	public byte[] getDecompositionStructure(int t, int c)
	{
		return defaultDecomposition;
	}
	
	public boolean isDyadic (int t, int c)
	{
		return (getDecompositionStructure(t, c) == null);
	}
	
}
