/* 
 * CVS identifier:
 * 
 * $Id: EntropyInformationCost.java,v 1.2 2005-11-15 14:06:05 dengel Exp $
 * 
 * Class:                   EntropyInformationCost
 * 
 * Description:             
 * 
 * Created on Feb 4, 2005
 * 
 * @author Dominik Engel <dengel@cosy.sbg.ac.at>
 * 
 * COPYRIGHT:
 * 
 * This software module was originally developed by Rapha�l Grosbois and
 * Diego Santa Cruz (Swiss Federal Institute of Technology-EPFL); Joel
 * Askel�f (Ericsson Radio Systems AB); and Bertrand Berthelot, David
 * Bouchard, F�lix Henry, Gerard Mozelle and Patrice Onno (Canon Research
 * Centre France S.A) in the course of development of the JPEG2000
 * standard as specified by ISO/IEC 15444 (JPEG 2000 Standard). This
 * software module is an implementation of a part of the JPEG 2000
 * Standard. Swiss Federal Institute of Technology-EPFL, Ericsson Radio
 * Systems AB and Canon Research Centre France S.A (collectively JJ2000
 * Partners) agree not to assert against ISO/IEC and users of the JPEG
 * 2000 Standard (Users) any of their rights under the copyright, not
 * including other intellectual property rights, for this software module
 * with respect to the usage by ISO/IEC and Users of this software module
 * or modifications thereof for use in hardware or software products
 * claiming conformance to the JPEG 2000 Standard. Those intending to use
 * this software module in hardware or software products are advised that
 * their use may infringe existing patents. The original developers of
 * this software module, JJ2000 Partners and ISO/IEC assume no liability
 * for use of this software module or modifications thereof. No license
 * or right to this software module is granted for non JPEG 2000 Standard
 * conforming products. JJ2000 Partners have full right to use this
 * software module for his/her own purpose, assign or donate this
 * software module to any third party and to inhibit third parties from
 * using this software module for non JPEG 2000 Standard conforming
 * products. This copyright notice must be included in all copies or
 * derivative works of this software module.
 * 
 * Copyright (c) 1999/2000 JJ2000 Partners.
 * 
 * 
 * 
 */

package jj2000.j2k.wavelet.analysis.bestbasis;

import jj2000.j2k.image.*;
import jj2000.j2k.util.ParameterList;
import jj2000.j2k.wavelet.Subband;
import java.util.HashMap;
import java.util.Iterator;
import java.lang.Integer;
import java.lang.Float;
import java.lang.Math;

public class EntropyX extends Entropy {

	public EntropyX(ParameterList pl) {
		super(pl);
	}

	/* 
	 * @see
	 * jj2000.j2k.wavelet.analysis.bestbasis.AddCostFunction#getCost(jj2000.
	 * j2k.image.DataBlkFloat, jj2000.j2k.wavelet.Subband)
	 */
	public double getCost(DataBlk datablk, Subband sb, String param) {
		double cost = super.getCost(datablk, sb, param);
		
		// Quantization has to be taken into account.
		String qType = pl.getParameter("Qtype");
		double qCost = 0; // derived
		if(qType.equals("reversible")){
			qCost = 8; // Set to 1 byte == 8 bit.
		}else if(qType.equals("expounded")){
			qCost = 16; // Set to 2 byte == 16 bit.
		}
		cost += qCost;
	
		int cbx = 64;
		int cby = 64;
		double cbpen = 15; // Set to almost two byte penalty.
		if (sb.w < cbx) {
			cost += cbpen * Math.ceil(cbx / sb.w);
		}
		if (sb.h < cby) {
			cost += cbpen * Math.ceil(cby / sb.h);
		}

		return cost;
	}
}
