/* 
 * CVS identifier:
 * 
 * $Id: Norml2.java,v 1.2 2005-11-15 14:06:05 dengel Exp $
 * 
 * Class:                   Norml2
 * 
 * Description:             
 * 
 * Created on Feb 4, 2005
 * 
 * @author Dominik Engel <dengel@cosy.sbg.ac.at>
 * 
 * COPYRIGHT:
 * 
 * This software module was originally developed by Rapha�l Grosbois and
 * Diego Santa Cruz (Swiss Federal Institute of Technology-EPFL); Joel
 * Askel�f (Ericsson Radio Systems AB); and Bertrand Berthelot, David
 * Bouchard, F�lix Henry, Gerard Mozelle and Patrice Onno (Canon Research
 * Centre France S.A) in the course of development of the JPEG2000
 * standard as specified by ISO/IEC 15444 (JPEG 2000 Standard). This
 * software module is an implementation of a part of the JPEG 2000
 * Standard. Swiss Federal Institute of Technology-EPFL, Ericsson Radio
 * Systems AB and Canon Research Centre France S.A (collectively JJ2000
 * Partners) agree not to assert against ISO/IEC and users of the JPEG
 * 2000 Standard (Users) any of their rights under the copyright, not
 * including other intellectual property rights, for this software module
 * with respect to the usage by ISO/IEC and Users of this software module
 * or modifications thereof for use in hardware or software products
 * claiming conformance to the JPEG 2000 Standard. Those intending to use
 * this software module in hardware or software products are advised that
 * their use may infringe existing patents. The original developers of
 * this software module, JJ2000 Partners and ISO/IEC assume no liability
 * for use of this software module or modifications thereof. No license
 * or right to this software module is granted for non JPEG 2000 Standard
 * conforming products. JJ2000 Partners have full right to use this
 * software module for his/her own purpose, assign or donate this
 * software module to any third party and to inhibit third parties from
 * using this software module for non JPEG 2000 Standard conforming
 * products. This copyright notice must be included in all copies or
 * derivative works of this software module.
 * 
 * Copyright (c) 1999/2000 JJ2000 Partners.
 * 
 * 
 * 
 */
package jj2000.j2k.wavelet.analysis.bestbasis;

import jj2000.j2k.image.*;
import jj2000.j2k.util.ParameterList;
import jj2000.j2k.wavelet.Subband;


/**
 * Compute the entropy of the local binary patterns.
 */
public class LBPECB extends CostFunction {

	public LBPECB(ParameterList pl) {
		super(pl);
	}

	/* return cost, this is for float data
	 * @see jj2000.j2k.wavelet.analysis.bestbasis.AddCostFunction#getCost(jj2000.j2k.image.DataBlkFloat, jj2000.j2k.wavelet.Subband)
	 */
	public double getCost(DataBlk datablk, Subband sb, String param) {
		double cost = 0;
				
		float data [];
		//Conversion, probably stupid?
		if ( datablk.getDataType() == datablk.TYPE_FLOAT )
		{
			data = ((DataBlkFloat)datablk).getDataFloat();
		}else{
			int idata[] = ((DataBlkInt)datablk).getDataInt();
			data = new float[idata.length];
			for( int i = 0; i<idata.length; i++)
				data[i] = (float)idata[i]; 
		}

		int nSyms = 256;
		int[] freqs = new int[nSyms];
		for(int i = 0; i < 256; i++) 
			freqs[i] = 0;

		int h = sb.h;
		int w = sb.w;
		int band_w = datablk.w;
		int band_h = datablk.h;
		int ulx = sb.ulx;
		int uly = sb.uly;
		int uoffset, offset, doffset;
		
		int lbpp;
		// LU U RU
		// L  C R
		// LD D RD
		float lu, u, ru, l, c, r, ld, d, rd;
		for(int y=1; y < h-1; y++) {
			uoffset = (uly+y-1)*band_w + ulx;
			offset = (uly+y)*band_w + ulx;
			doffset = (uly+y+1)*band_w + ulx;
			for(int x=1; x < w-1; x++) {
				lbpp = 0;
				lu = Math.round(data[uoffset + x - 1]/32);
				u = Math.round(data[uoffset + x]/32);
				ru = Math.round(data[uoffset + x + 1]/32);
				l = Math.round(data[offset + x - 1]/32);
				c = Math.round(data[offset + x]/32);
				r = Math.round(data[offset + x + 1]/32);
				ld = Math.round(data[doffset + x - 1]/32);
				d = Math.round(data[doffset + x]/32);
				rd = Math.round(data[doffset + x + 1]/32);

				if(lu < c)
					lbpp += 1;
				if(u < c)
					lbpp += 2;
				if(ru < c)
					lbpp += 4;
				if(l < c)
					lbpp += 8;
//				if(r < c)
//					lbpp += 16;
//				if(ld < c)
//					lbpp += 32;
//				if(d < c)
//					lbpp += 64;
//				if(rd < c)
//					lbpp += 128;
//
				freqs[lbpp]++;

			}		
		}		
        
		double entropy = 0;
		double p;
		for(int i = 0; i < 256; i++)
		{
			p = freqs[i];
			p /= w * h;
			//System.out.println("p is " + p);
			if(p != 0)
				entropy += p * Math.log(1./p);
		}

		//cost = entropy * w * h;
		cost = entropy * w * h;
		// If the subband is smaller than the codeblock size 
		// there has to be some penaltiy
		int cbx = 64;
		int cby = 64;
		float cbpen = 8;
		if(w < cbx){
			cost += cbpen * Math.ceil(cbx / w);
		}
		if(h < cby){
			cost += cbpen * Math.ceil(cby / h);
		}


//		System.out.println("The LBPE cost is " + cost);
		return cost;		
	}

}
