/* 
 * CVS identifier:
 * 
 * $Id: Norml.java,v 1.2 2005-11-15 14:06:05 dengel Exp $
 * 
 * Class:                   Norml
 * 
 * Description:             
 * 
 * Created on Feb 4, 2005
 * 
 * @author Dominik Engel <dengel@cosy.sbg.ac.at>
 * 
 * COPYRIGHT:
 * 
 * This software module was originally developed by Rapha�l Grosbois and
 * Diego Santa Cruz (Swiss Federal Institute of Technology-EPFL); Joel
 * Askel�f (Ericsson Radio Systems AB); and Bertrand Berthelot, David
 * Bouchard, F�lix Henry, Gerard Mozelle and Patrice Onno (Canon Research
 * Centre France S.A) in the course of development of the JPEG2000
 * standard as specified by ISO/IEC 15444 (JPEG 2000 Standard). This
 * software module is an implementation of a part of the JPEG 2000
 * Standard. Swiss Federal Institute of Technology-EPFL, Ericsson Radio
 * Systems AB and Canon Research Centre France S.A (collectively JJ2000
 * Partners) agree not to assert against ISO/IEC and users of the JPEG
 * 2000 Standard (Users) any of their rights under the copyright, not
 * including other intellectual property rights, for this software module
 * with respect to the usage by ISO/IEC and Users of this software module
 * or modifications thereof for use in hardware or software products
 * claiming conformance to the JPEG 2000 Standard. Those intending to use
 * this software module in hardware or software products are advised that
 * their use may infringe existing patents. The original developers of
 * this software module, JJ2000 Partners and ISO/IEC assume no liability
 * for use of this software module or modifications thereof. No license
 * or right to this software module is granted for non JPEG 2000 Standard
 * conforming products. JJ2000 Partners have full right to use this
 * software module for his/her own purpose, assign or donate this
 * software module to any third party and to inhibit third parties from
 * using this software module for non JPEG 2000 Standard conforming
 * products. This copyright notice must be included in all copies or
 * derivative works of this software module.
 * 
 * Copyright (c) 1999/2000 JJ2000 Partners.
 * 
 * 
 * 
 */

package jj2000.j2k.wavelet.analysis.bestbasis;

import jj2000.j2k.image.*;
import jj2000.j2k.util.ParameterList;
import jj2000.j2k.wavelet.Subband;


/**
 * @author dengel
 *
 */
public class ThreshP extends CostFunction {

	public ThreshP(ParameterList pl) {
		super(pl);
	}

	/* return cost, this is for float data
	 * @see jj2000.j2k.wavelet.analysis.bestbasis.AddCostFunction#getCost(jj2000.j2k.image.DataBlkFloat, jj2000.j2k.wavelet.Subband)
	 */
	public double getCost(DataBlk datablk, Subband sb, String param) {
		double q, THRESHOLD = 16.f;
		try {
			q = (new Double(param)).doubleValue();
		} catch (NullPointerException e) {
			q = THRESHOLD;
		} catch (NumberFormatException e) {
			e.printStackTrace();
			throw e;
		}
	    THRESHOLD = q;
		double cost = 0;
				
		float data [];
		
		if ( datablk.getDataType() == datablk.TYPE_FLOAT )
		{
			data = ((DataBlkFloat)datablk).getDataFloat();
		}else{
			int idata[] = ((DataBlkInt)datablk).getDataInt();
			data = new float[idata.length];
			for( int i = 0; i<idata.length; i++)
				data[i] = (float)idata[i]; 
		}
		int h = sb.h;
		int w = sb.w;
		int band_w = datablk.w;
		int band_h = datablk.h;
		int ulx = sb.ulx;
		int uly = sb.uly;
		int offset;
		
		
		for(int i=0; i<h; i++) {
			offset = (uly+i)*band_w + ulx;
			for(int j=0; j<w; j++) {
				if (Math.abs(data[offset + j]) < THRESHOLD)
				{	
					cost++;
				}
			}		
		}		
          
		return w * h - cost;		
		
	}

}
