/* 
 * Simple cost estimation by counting zeros.
 * Parameter the quantization value / divisor.
 * 
 */

package jj2000.j2k.wavelet.analysis.bestbasis;

import jj2000.j2k.image.*;
import jj2000.j2k.util.ParameterList;
import jj2000.j2k.wavelet.Subband;

public class ThreshPCB extends CostFunction {

	public ThreshPCB(ParameterList pl) {
		super(pl);
	}

	/*
	 * return cost, this is for float data
	 * 
	 * @see
	 * jj2000.j2k.wavelet.analysis.bestbasis.AddCostFunction#getCost(jj2000.
	 * j2k.image.DataBlkFloat, jj2000.j2k.wavelet.Subband)
	 */
	public double getCost(DataBlk datablk, Subband sb, String param) {
		double q, THRESHOLD = 16.f;
		try {
			q = (new Double(param)).doubleValue();
		} catch (NullPointerException e) {
			q = THRESHOLD;
		} catch (NumberFormatException e) {
			e.printStackTrace();
			throw e;
		}
		THRESHOLD = q;

		double cost = 0;

		float data[];

		if (datablk.getDataType() == datablk.TYPE_FLOAT) {
			data = ((DataBlkFloat) datablk).getDataFloat();
		} else {
			int idata[] = ((DataBlkInt) datablk).getDataInt();
			data = new float[idata.length];
			for (int i = 0; i < idata.length; i++)
				data[i] = (float) idata[i];
		}

		int h = sb.h;
		int w = sb.w;
		int band_w = datablk.w;
		int band_h = datablk.h;
		int ulx = sb.ulx;
		int uly = sb.uly;
		int offset;

		for (int i = 0; i < h; i++) {
			offset = (uly + i) * band_w + ulx;
			for (int j = 0; j < w; j++) {
				if (Math.abs(data[offset + j]) < THRESHOLD) {
					cost++;
				}
			}
		}

		cost = w * h - cost;
		int cbx = 64;
		int cby = 64;
		float cbpen = 15;
		if (w < cbx) {
			cost += cbpen * Math.ceil(cbx / w);
		}
		if (h < cby) {
			cost += cbpen * Math.ceil(cby / h);
		}

		return cost;
	}
}
