/* 
 * CVS identifier:
 * 
 * $Id: SymbolicMatrix.java,v 1.6 2006-03-28 09:32:51 dengel Exp $
 * 
 * Class:                   Symbolic Matrix
 * 
 * Description:             
 * 
 * Created on June 28, 2005
 * 
 * @author Dominik Engel <dengel@cosy.sbg.ac.at>
 * 
 * COPYRIGHT:
 * 
 * This software module was originally developed by Raphal Grosbois and
 * Diego Santa Cruz (Swiss Federal Institute of Technology-EPFL); Joel
 * Askelf (Ericsson Radio Systems AB); and Bertrand Berthelot, David
 * Bouchard, Flix Henry, Gerard Mozelle and Patrice Onno (Canon Research
 * Centre France S.A) in the course of development of the JPEG2000
 * standard as specified by ISO/IEC 15444 (JPEG 2000 Standard). This
 * software module is an implementation of a part of the JPEG 2000
 * Standard. Swiss Federal Institute of Technology-EPFL, Ericsson Radio
 * Systems AB and Canon Research Centre France S.A (collectively JJ2000
 * Partners) agree not to assert against ISO/IEC and users of the JPEG
 * 2000 Standard (Users) any of their rights under the copyright, not
 * including other intellectual property rights, for this software module
 * with respect to the usage by ISO/IEC and Users of this software module
 * or modifications thereof for use in hardware or software products
 * claiming conformance to the JPEG 2000 Standard. Those intending to use
 * this software module in hardware or software products are advised that
 * their use may infringe existing patents. The original developers of
 * this software module, JJ2000 Partners and ISO/IEC assume no liability
 * for use of this software module or modifications thereof. No license
 * or right to this software module is granted for non JPEG 2000 Standard
 * conforming products. JJ2000 Partners have full right to use this
 * software module for his/her own purpose, assign or donate this
 * software module to any third party and to inhibit third parties from
 * using this software module for non JPEG 2000 Standard conforming
 * products. This copyright notice must be included in all copies or
 * derivative works of this software module.
 * 
 * Copyright (c) 1999/2000 JJ2000 Partners.
 * 
 * 
 * 
 */

package jj2000.j2k.wavelet.synthesis;

import java.io.BufferedOutputStream;
import java.io.DataOutputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import jj2000.j2k.util.FacilityManager;
import jj2000.j2k.util.MsgLogger;

/**
 * @author dengel
 * 
 *  <<Singleton>>
 */
public class SymbolicMatrix {

	public String data[]; //symbolic data
	public String buf[]; // buffer
	public String coeff[]; //the transform coefficients
	public String img[]; //the reconstructed image

	int w, h;

	public static SymbolicMatrix instance = null;

	/**
	 * create SymbolicMatrix of size w, h
	 * 
	 * @param w
	 * @param h
	 */
	public SymbolicMatrix(int w, int h) {
		FacilityManager.getMsgLogger().printmsg(MsgLogger.INFO, "Symbolic Matrix initialized");
		data = new String[w * h];
		coeff = new String[w * h];
		img = new String[w * h];
		this.w = w;
		this.h = h;
		/*for (int j = 0; j < h; j ++)
			for (int i = 0; i < w; i ++)
			{
				// Mathematica does not use zero-indexing
				// Furthermore, conceptually, indexing is done with l, r (vertical first)
				// rather than x, y (horizontal first)
				data[j*w + i] = "c[[" + (j+1) + "," + (i+1) + "]]";
			}*/
		

		for (int i = 0; i < w*h; i ++)
			{
				// Mathematica does not use zero-indexing
				//data[j*w + i] = "c[[" + (j+1) + "," + (i+1) + "]]";
				data[i] = "c[[" + (i+1) + "]]";
			}
	}
	
	/**
	 * ensure only one SymbolicMatrix exists
	 * 
	 * @param w
	 * @param h
	 */
	public static SymbolicMatrix createInstance(int w, int h) {
		if (instance == null)
			instance = new SymbolicMatrix(w, h);
		return instance;
	}

	/**
	 * get the global instance of SymbolicMatrix
	 * 
	 * @return global instance of SymbolicMatrix
	 */
	public static SymbolicMatrix getInstance() {
		return instance;
	}
	
	public void createLineBuffer (int line)
	{
		buf = new String[w];
		for (int i = 0; i < w; i++)
		{
			buf[i] = data[(line*w) + i];
		}
	}
		
	public void createRowBuffer (int row)
	{
		buf = new String[h];
		for (int j = 0; j < h; j++)
		{
			buf[j] = data[(j*w) + row];
		}
	}
	
	public String[] getBuffer ()
	{
		return buf;
	}

	public String toString ()
	{
		String tmp = "";
		for (int i = 0; i < w*h; i ++)
		{
			if (i % w == 0) tmp += "\n";
			tmp += data[i] + ", ";
		}
		return tmp;
	}
	
	public String printBuffer ()
	{
		if (buf == null) return "<null>";
		String tmp = "";
		for (int i = 0; i < w; i ++)
		{
			tmp += buf[i] + ", ";
		}
		return tmp;
	}
	
	public void collectCoefficients (float[] coeff_data)
	{
		for (int i = 0; i < w*h; i ++ )
		{
			coeff[i]  = Float.toString(coeff_data[i]);
		}
	}
	
	public void collectImageData (float[] img_data) 
	{
		for (int i = 0; i < w*h; i ++ )
		{
			img[i]  = Float.toString(img_data[i]);
		}
	}
	
	
	
	//write collected Coefficients to File fname for Mathematica
	public void writeCoefficients (String fname)
	{		
		try {
			//create a buffered output stream
			FileOutputStream fos = new FileOutputStream (fname);
			BufferedOutputStream bout = new BufferedOutputStream (fos);
			DataOutputStream out = new DataOutputStream(bout);
							
/*			out.writeBytes("c = {\n");
			for (int j = 0; j < h; j ++ )
			{
				out.writeBytes("{");
				for (int i = 0; i < w; i ++ )				
				{	
					out.writeBytes ("" + coeff[j*w + i]);
					if (i != w-1) out.writeBytes(", ");
				}
				out.writeBytes("}");
				if (j != h-1) out.writeBytes(",");
				out.writeBytes("\n");
			}
			out.writeBytes("};");*/
			out.writeBytes("c = {\n");
			for (int i = 0; i < w*h; i ++ )				
				{	
					out.writeBytes ("" + coeff[i]);
					if (i != w*h-1) out.writeBytes(", ");
				}
				out.writeBytes("\n};\n");							
			out.flush();
			out.close();
		}
		catch (FileNotFoundException fnfe) {
			FacilityManager.getMsgLogger().printmsg(MsgLogger.ERROR, "File not Found"+fnfe.getMessage());
		}
		catch (IOException ioe) {			
			FacilityManager.getMsgLogger().printmsg(MsgLogger.ERROR, "IO Error");	
		}
		finally
		{
			FacilityManager.getMsgLogger().printmsg(MsgLogger.INFO, "Cofficients for Mathematica written to " + fname);
		}
	}

	
	//write symbolic data to File fname for Mathematica
	public void writeSymData (String fname)
	{		
		try {
			//create a buffered output stream
			FileOutputStream fos = new FileOutputStream (fname);
			BufferedOutputStream bout = new BufferedOutputStream (fos);
			DataOutputStream out = new DataOutputStream(bout);
			
			/*ALPHA = alpha;
	    	BETA = (float) (-1 / ( 4 * Math.pow( 1 + 2 * ALPHA , 2) ));
	    	GAMMA = (float) ((-1-4*ALPHA-4* Math.pow(ALPHA,2))/(1+4*ALPHA));
	    	DELTA = (float) (( 4 - ((2 + 4*ALPHA)/Math.pow(1 + 2*ALPHA, 4)) + ((1 - 8*ALPHA)/Math.pow(1 + 2*ALPHA, 2)) ) / 16  );
	    	KH = (float) (  2 * Math.sqrt(2) * (1 + 2* ALPHA)  / (1+4*ALPHA)  );
	    	KL = 1/KH;*/
			
			out.writeBytes ("(* a = -1.6; *)\n");
	    	out.writeBytes ("b = -1 / ( 4 * ( 1 + 2 * a)^2 );\n");	    	
	    	out.writeBytes ("g = (-1-4*a-4* a^2)/(1+4*a);\n");
	    	out.writeBytes ("d =  (4 - (2 + 4*a) / (1 + 2*a)^4 + (1 - 8*a)/(1 + 2*a)^2 ) / 16;\n");
	    	out.writeBytes ("h =  2 * Sqrt[2] * (1 + 2* a)  / (1+4*a);\n");
	    	out.writeBytes ("l = 1/h;\n");

			
	    	
	    	
			out.writeBytes("s = {\n");			
				for (int i = 0; i < w*h; i ++ )				
				{	
					out.writeBytes (data[i]);
					if (i != w*h-1) out.writeBytes(",\n");
				}
				out.writeBytes("\n};\n");
			
			/*out.writeBytes("s = {\n");
			for (int j = 0; j < h; j ++ )
			{
				out.writeBytes("{");
				for (int i = 0; i < w; i ++ )				
				{	
					out.writeBytes (data[j*w + i]);
					if (i != w-1) out.writeBytes(", ");
				}
				out.writeBytes("}");
				if (j != h-1) out.writeBytes(",");
				out.writeBytes("\n");
			}
			out.writeBytes("}");*/
			
		
			out.flush();
			out.close();
		}
		catch (FileNotFoundException fnfe) {
			FacilityManager.getMsgLogger().printmsg(MsgLogger.ERROR, "File not Found"+fnfe.getMessage());
		}
		catch (IOException ioe) {			
			FacilityManager.getMsgLogger().printmsg(MsgLogger.ERROR, "IO Error");	
		}
		finally
		{
			FacilityManager.getMsgLogger().printmsg(MsgLogger.INFO, "Symbolic Data for Mathematica written to " + fname);
		}
	}


	//write reconstructed Image data to File fname for Mathematica
	public void writeImageData (String fname)
	{		
		try {
			//create a buffered output stream
			FileOutputStream fos = new FileOutputStream (fname);
			BufferedOutputStream bout = new BufferedOutputStream (fos);
			DataOutputStream out = new DataOutputStream(bout);
							
			/*out.writeBytes("i = {\n");
			for (int j = 0; j < h; j ++ )
			{
				out.writeBytes("{");
				for (int i = 0; i < w; i ++ )				
				{	
					out.writeBytes ("" + img[j*w + i]);
					if (i != w-1) out.writeBytes(", ");
				}
				out.writeBytes("}");
				if (j != h-1) out.writeBytes(",");
				out.writeBytes("\n");
			}
			out.writeBytes("};");*/
			
			out.writeBytes("i = {\n");
				for (int i = 0; i < w*h; i ++ )				
				{	
					out.writeBytes ("" + img[i]);
					if (i != w*h-1) out.writeBytes(", ");
				}			
			out.writeBytes("};\n");
						
		
			out.flush();
			out.close();
		}
		catch (FileNotFoundException fnfe) {
			FacilityManager.getMsgLogger().printmsg(MsgLogger.ERROR, "File not Found"+fnfe.getMessage());
		}
		catch (IOException ioe) {			
			FacilityManager.getMsgLogger().printmsg(MsgLogger.ERROR, "IO Error");	
		}
		finally
		{
			FacilityManager.getMsgLogger().printmsg(MsgLogger.INFO, "Cofficients for Mathematica written to " + fname);
		}
	}
	
	public void writeImageAsPGM(String fname)
	{	
		try {
			//create a buffered output stream
			FileOutputStream fos = new FileOutputStream (fname);
			BufferedOutputStream bout = new BufferedOutputStream (fos);
			DataOutputStream out = new DataOutputStream(bout);
			
			//Write PGM Header
			out.writeBytes("P2\n" + w + " " + h + "\n255\n");
			
			float val;
			//write data, shifting values back to original range 0..255
			for (int j = 0; j < h; j++)
			{
				for(int i = 0; i < w; i++)
				{

					val = Float.parseFloat(img[j*w + i]);
					
					//val = val & 0xFF;
					
					//Shift back to original range
					val = val + 128; 
					val = (val < 0) ? 0 : (val > 255) ? 255 : val; //check range
					/*if ((val < 0) || (val > 255))
					{
						FacilityManager.getMsgLogger().printmsg(MsgLogger.WARNING, "val = " + val);
					}*/
					out.writeBytes (Integer.toString( Math.round(val)) + " " );
				}
				out.writeBytes("\n");
			}
			out.flush();
			out.close();
		}
		catch (FileNotFoundException fnfe) {
			FacilityManager.getMsgLogger().printmsg(MsgLogger.ERROR, "File not Found+fnfe.getMessage())");
		}
		catch (IOException ioe) {			
			FacilityManager.getMsgLogger().printmsg(MsgLogger.ERROR, "IO Error");	
		}
		finally
		{
			FacilityManager.getMsgLogger().printmsg(MsgLogger.INFO, "Written to " + fname);
		}
	
	}		
	
}